<blockquote style="background-color: yellow; border-color: black"><img alt="security goose says" title="HONK" src="{{ "third_party/shiny_goose.png" | relURL }}" style="max-width: 6em; float: right;" /><p style="font-weight: bold; text-decoration: underline; padding: 0; margin: 0; margin-top: -.25em; font-size: 1.25em;">Security Goose Says:</p>{{ .Inner | markdownify }}</blockquote>
