html {
  width: 100%;
  font-size: 18px;
  line-height: 1;
  -webkit-font-smoothing: antialiased !important;
  -moz-osx-font-smoothing: grayscale !important;
  text-rendering: optimizeLegibility !important;
  -webkit-tap-highlight-color: transparent;
  height: 100%;
  min-height: 100%;
  margin: 0;
  padding: 0;
  font-size: 100%;
  background-color: #326ce5;
  color: white;
  font-family: -apple-system, BlinkMacSystemFont, Roboto, Droid Sans, Helvetica Neue, Helvetica, Arial, sans-serif, Apple Color Emoji, Segoe UI Emoji, Segoe UI Symbol, Noto Color Emoji;
}

code,
pre {
  font-family: SFMono-Regular, Consolas, Liberation Mono, Menlo, Roboto Mono, Courier, monospace;
}

body {
  width: 100%;
  margin: 0;
  padding: 0;
  font-size: 18px;
  line-height: 1.5;
  min-height: 100%;
  /* sticky footer */
  display: flex;
  flex-direction: column;
  color: black;
  background-color: white;
}

#wrapper {
  background-color: white;
  max-width: 100%;
  clear: both;
  overflow-x: hidden;
  flex: 1 0 auto;
  margin: 0;
  margin-left: 240px;
}

.sidebar-collapsed #wrapper {
  margin-left: 0;
}

#content,
#footer-content {
  margin: 0 auto;
  padding: 0 1em;
  width: calc(100% - 2em);
  max-width: 40em;
}

#footer {
  background: #326ce5;
  color: white;
  padding: 1em;
  margin-top: 1em;
  border-top: .1em solid rgba(0, 0, 0, 0.1);
}

#footer a {
  color: white;
}

/* page footer*/
.footer {
  text-align: center;
}

.footer>* {
  font-size: .8em;
  line-height: 1.4;
}

#content {
  /* clear navbar */
  padding-top: 1.5em;
  /* push down footer */
  min-height: calc(100vh - 4em);
}

#navbar {
  position: fixed;
  top: 0;
  width: calc(100% - 240px);
  padding: 2px 0;
  margin-left: .1em;
  /*background-color: #326ce5;*/
  background-color: #326ce5;
  border-bottom: .1em solid rgba(0, 0, 0, 0.1);
  display: flex;
  justify-content: space-between;
  z-index: 99999;
}

.sidebar-collapsed #navbar {
  width: 100%;
  margin-left: 0;
}

#navbar-title>a {
  font-weight: bold;
  font-size: 1.2em;
  padding: 0 4px;
  vertical-align: middle;
}

#navbar-title>a,
#github>a {
  text-decoration: none;
  color: white;
}

#navbar>#github {
  filter: invert(1);
}

#sidebar-toggle {
  color: white;
  cursor: pointer;
  padding: .1em .5em;
  padding-bottom: 0;
  user-select: none;
}

#github {
  padding: 0 .5em;
}

#github img {
  height: 1.4em;
  width: 1.4em;
  vertical-align: middle;
}

#noticebar {
  padding: 1.5em;
  padding-top: 2em;
  padding-bottom: 1em;
  margin-bottom: -1.5em;
  border-bottom: .1em solid rgba(0, 0, 0, 0.1);
}

#noticebar>* {
  max-width: 46rem;
  margin-left: auto;
  margin-right: auto;
  margin-block-end: 0;
  margin-block-start: 0.25em;
}

#sidebar {
  height: 100%;
  position: fixed;
  display: flex;
  flex-direction: column;
  width: 240px;
  border-right: .1em solid rgba(0, 0, 0, 0.1);
  background-color: #fafafa;
  z-index: 99999;
  overflow-y: auto;
}

.sidebar-collapsed #sidebar {
  display: none;
}

#sidebar-logo {
  padding: 16px 16px;
  padding-bottom: 10px;
}

#sidebar-title {
  display: none;
  margin-top: -16px;
  padding-right: 37px;
  margin-bottom: 10px;
  text-align: center;
  text-decoration: none;
  font-size: 24px;
  font-weight: bold;
}

#sidebar li.active-entry :first-child,
#sidebar li.active :first-child {
  font-weight: bold;
}

#sidebar li.active-entry :first-child {
  color: black;
}

/* consistent link color */
a {
  color: #326be5;
}

p {
  margin-block-start: .5em;
}

h1,
h2,
h3,
h4,
h5 {
  margin-block-start: 1rem;
  margin-block-end: .25rem;
}

/* ensure images don't go off the page */
#content img {
  max-width: 100%;
}

hr {
  background: #333;
  border: 1px solid #333;
}

/* code block styling */
pre {
  background-color: #f3f4f4 !important;
  overflow-x: auto;
  border-radius: 0;
  padding: 1em;
  font-size: 85%;
}

code {
  background-color: #f3f4f4;
  border-radius: 3px;
  padding: .1em .4em;
  font-size: 85%;
}

pre code {
  background-color: transparent;
  padding: 0;
  font-size: 100%;
  border-radius: 0;
}

/* inline copyable code snippet styling */
table.includecode {
  max-width: 100%;
  width: 100%;
  table-layout: fixed;
  border: 1px solid rgba(0, 0, 0, .125);
  border-spacing: 0;
}

table.includecode thead th {
  border-bottom: 1px solid rgba(0, 0, 0, .125);
  font-weight: normal;
  font-size: 85%;
}

table.includecode thead th {
  text-align: right;
}

table.includecode thead th a {
  font-family: monospace;
}

table.includecode thead th button {
  vertical-align: middle;
  background: transparent;
  border: 0;
  cursor: pointer;
}

table.includecode tbody {
  background: #f3f4f4;
}

table.includecode pre {
  margin: 0;
}

/* don't display this, these are textareas needed to copy to clipboard */
.hidden-copy-text {
  background: transparent;
  width: 2em;
  height: 2em;
  position: fixed;
  top: 0;
  left: 0;
}

/* heading anchors */
.hanchor {
  font-size: 100%;
  visibility: hidden;
  font-size: .5em;
  vertical-align: middle;
  text-decoration: none !important;
}

h1:hover a,
h2:hover a,
h3:hover a,
h4:hover a {
  visibility: visible;
}

#content #TableOfContents ul {
  margin-block-start: 0;
  margin-block-end: 0;
}

#content ul {
  padding-left: 1.75em;
  margin-block-start: .5em;
  margin-block-end: .5em;
}

#content ul ul {
  padding-left: 1em;
  margin-block-start: 0;
}

#sidebar ul {
  margin-bottom: 1em;
}

#sidebar ul ul {
  padding-left: 1.25em;
}

.page-description {
  border-left-color: black;
  background: #3b4d56;
  color: white;
  font-size: 1.2em;
  line-height: 1.4em;
  margin-block-start: 0;
  margin-block-end: 0;
}

.page-description a {
  color: #0bd8ec;
}

.page-description code {
  color: black;
}

blockquote {
  border-left: .5rem solid #326ce4;
  background: #c9e6ff;
  color: black;
  padding: 1em 1.5em;
  margin-inline-start: 0;
  margin-inline-end: 0;
}

blockquote p {
  margin-block-start: 0.75em;
  margin-block-end: 0.75em;
}

/* style video embeds to be full-width in content */
.video-wrapper {
  position: relative;
  width: 100%;
  height: 0;
  padding-bottom: 56.25%;
}

.video-wrapper iframe {
  position: absolute;
  top: 0;
  left: 0;
  width: 100%;
  height: 100%;
}

/* mock github labels */
.gh-label {
  /* mimic github */
  display: inline-block;
  padding: 0 7px;
  font-size: 12px;
  font-weight: 500;
  line-height: 18px;
  border: 1px solid transparent;
  border-radius: 2em;
  background: #bfd4f2;
  /* additional styling to allow using on a tags */
  text-decoration: none;
}

/* mobile */
@media (max-width: 50em) {
  #content {
    font-size: 16px;
  }

  #wrapper {
    margin-left: 0;
  }

  #wrapper #navbar {
    margin-left: 241px;
  }

  .sidebar-collapsed #wrapper #navbar {
    margin-left: 0;
  }

  #content ul {
    padding-left: 1.5em;
  }
}

:target::before {
  content: "";
  display: block;
  height: 2em;
  margin: -2em 0 0;
}

/* https://github.com/adityatelange/hugo-PaperMod/issues/828#issuecomment-1171994855 */
/* Fixes iOS font sizing anomaly */
code {
  text-size-adjust: 100%;
  -ms-text-size-adjust: 100%;
  -moz-text-size-adjust: 100%;
  -webkit-text-size-adjust: 100%;
}